import React, { Component } from 'react'
import ReactDOM from 'react-dom'
import _ from 'lodash'
import MenuItem from 'material-ui/MenuItem'
import Popover from 'material-ui/Popover'
import Menu from 'material-ui/Menu'
import TextField from 'material-ui/TextField'
import Chip from 'material-ui/Chip'
import keycode from 'keycode'

let hrWithOutTransition = {
  borderStyle: 'none none solid',
  borderBottomWidth: '2px',
  borderColor: 'rgb(0, 188, 212)',
  boxSizing: 'content-box',
  margin: '0px',
  width: '100%',
  transform: 'scaleX(0)',
  transition: 'all 450ms cubic-bezier(0.23, 1, 0.32, 1) 0ms',
  backgroundColor: 'white',
}

let hrTransition = {
  transform: 'scaleY(1)'
}

let inputHr = {
  borderStyle: 'none none solid',
  borderBottomWidth: '1px',
  borderColor: 'rgb(224, 224, 224)',
  boxSizing: 'content-box',
  margin: '0px',
  width: '100%',
  marginTop: '-8px',
}

export default class MultiSelect extends Component {
  constructor(props) {
    super(props);

    this.state = {
      suggestions: this.props.suggestionItems,
      selectedItems: this.props.selectedItems,
      currentText: '',
      isOpen: false,
      focusTextField: true,
      showTransition: false,
      filterSuggestions: []
    }
  }

  handleRequestDelete = (value) => {

    this.selectedItems = this.state.selectedItems;
    const itemToDelete = this.selectedItems.map((item) => item.value).indexOf(value);
    this.selectedItems.splice(itemToDelete, 1);
    this.setState({selectedItems: this.selectedItems}); 
  }

  handleFocus = () => {
    
    this.setState({
      anchorEl: ReactDOM.findDOMNode(this.refs.inputFieldCont),
      showTransition: true,
      focusTextField: true,
      filterSuggestions: []
    })
  }

  handleBlur = () => {
    if (this.state.focusTextField) {
      this.close()

      this.setState({
        focusTextField: false
      })
    } 
  }

  close = () => {
    
    this.setState({
      isOpen: false,
      anchorEl: null,
      showTransition: false
    })
  } 

  displayAll = () => {
    this.setState({
      filterSuggestions: this.state.suggestions,
      isOpen: true
    })
  }

  handleKeyDown = (event) => {
    if (keycode(event) === 'enter') {
      this.displayAll()  
    }
  }

  filterData = (value) => {
    return this.state.suggestions.filter((suggestion) => {
      return _.includes(suggestion.value, value)
    })
  }

  handleChangeText = (e, value) => {
    
    if (!_.isEmpty(this.filterData(value)) && !this.state.isOpen) {

      this.setState({
        isOpen: true
      })
    }
    
    this.setState({
      filterSuggestions: this.filterData(value),
      currentText: value
    })
  }

  closeDropdown = () => {

    this.setState({
      isOpen: false    
    })
  }

  onSelection = (event, menuItem, index) => {

   // this.props.addToSelectedItems(event, menuItem.props.value, index)

    this.setState({
      currentText: ''
    })

    this.refs['textInput'].focus()
  }

  render() {

    let {hintText} = this.props
    let {currentText, isOpen, selectedItems, focusTextField, showTransition, anchorEl, filterSuggestions} = this.state

    let hrClasses = showTransition ? _.merge(hrWithOutTransition, _.clone(hrTransition)) : hrWithOutTransition

    return (
      <div>
        <div className="dropdownContainer" style={{width: '80%'}}>
            <div ref='inputFieldCont' style={{width: '100%', marginTop: '19px', display: 'flex', alignItems: 'center', paddingBottom: 4}}>
                 
              {selectedItems.map((item, key) => {
                 
                return (
                  <Chip
                    key={key}
                    onRequestDelete={() => this.handleRequestDelete(item.value)}
                    style={{margin: 3}}
                  >
                    {item.label}
                  </Chip>
                )
              })} 

              <TextField 
                id="inputField"
                ref='textInput'
                value={currentText}
                autoComplete="off"
                hintText={_.capitalize(hintText)} 
                style={{width: '22%', marginRight: 2}}
                onKeyDown={this.handleKeyDown}
                onChange={this.handleChangeText}
                onFocus={this.handleFocus}
                onBlur={this.handleBlur}
                underlineShow={false} 
              />
          </div>

          <hr style={inputHr} />
          <hr style={hrClasses} />
        </div>
        <Popover 
          open={isOpen}
          useLayerForClickAway={false}
          style={{width: '50%'}}
          anchorEl={anchorEl}
          anchorOrigin={{horizontal: 'left', vertical: 'bottom'}}
          targetOrigin={{horizontal: 'left', vertical: 'top'}}
          onRequestClose={this.closeDropdown}
        >
          <Menu 
            multiple={_.isArray(selectedItems)}
            onItemTouchTap={this.onSelection}
            disableAutoFocus={focusTextField}
            desktop={true}
            value={selectedItems}
            maxHeight={250} 
            style={{overflowY: 'auto', overflowX: 'auto'}}
          >
            {
              filterSuggestions.map((suggestion, i) => {
                return <MenuItem key={i} primaryText={suggestion.label} value={suggestion}/>
              })
            }
          </Menu>
        </Popover>
      </div>
    );
  }
}
